import axios from "axios";
import {tansParams, blobValidate} from "./tools";
import cache from "./cache";
import {saveAs} from "file-saver";

let downloadLoadingInstance;
// 是否显示重新登录
export let isRelogin = {show: false};

axios.defaults.headers["Content-Type"] = "application/json;charset=utf-8";
// 创建axios实例
const service = axios.create({
    // 超时
    timeout: 1000000,
});

// request拦截器
service.interceptors.request.use(
    (config) => {
        // console.log('axios request...', config);
        // 是否需要防止数据重复提交
        const isRepeatSubmit = (config.headers || {}).repeatSubmit === false;
        // get请求映射params参数
        if (config.method === "get" && config.params) {
            let url = config.url + "?" + tansParams(config.params);
            url = url.slice(0, -1);
            config.params = {};
            config.url = url;
        }
        if (
            !isRepeatSubmit &&
            (config.method === "post" || config.method === "put")
        ) {
            const requestObj = {
                url: config.url,
                data:
                    typeof config.data === "object"
                        ? JSON.stringify(config.data)
                        : config.data,
                time: new Date().getTime(),
            };
            const sessionObj = cache.session.getJSON("sessionObj");
            if (
                sessionObj === undefined ||
                sessionObj === null ||
                sessionObj === ""
            ) {
                cache.session.setJSON("sessionObj", requestObj);
            } else {
                const s_url = sessionObj.url; // 请求地址
                const s_data = sessionObj.data; // 请求数据
                const s_time = sessionObj.time; // 请求时间
                const interval = 0; // 间隔时间(ms)，小于此时间视为重复提交
                if (
                    s_data === requestObj.data &&
                    requestObj.time - s_time < interval &&
                    s_url === requestObj.url
                ) {
                    const message = "数据正在处理，请勿重复提交";
                    console.warn(`[${s_url}]: ` + message);
                    return Promise.reject(new Error(message));
                } else {
                    cache.session.setJSON("sessionObj", requestObj);
                }
            }
        }
        return config;
    },
    (error) => {
        console.log(error);
        Promise.reject(error);
    }
);

// 响应拦截器
service.interceptors.response.use(
    (res) => {
        // 未设置状态码则默认成功状态
        const code = res.status || 200;
        if (res.config.responseType === 'blob') {
            return res
        }
        if (res.request.responseType === "arraybuffer"
        ) {
            let enc = new TextDecoder("utf-8");
            let json = null
            try {
                json = JSON.parse(enc.decode(new Uint8Array(res.data)));
            } catch (e) {
                json = res.data
            }
            return json
        }
        if (code === 401) {
            if (!ELEMENT.$_overTime) {
                ELEMENT.$_overTime = ELEMENT.Notification.error({
                    title: "失败",
                    message: "code 401, 3秒后退出登录",
                });
                setTimeout(() => {
                    window.location.href = "/";
                }, 3000);
            }
            return Promise.reject("无效的会话，或者会话已过期，请重新登录。");
        }

        if (code == 200) {
            if (res.config.showMsg) {
                ELEMENT.Message.success(res.data.message || res.data.status);
            }
            return res.data;
        } else {
            if (res.config.showMsg) {
                ELEMENT.Message.error(res.data.message || res.data.status);
            }
            return null
        }
    },
    (error) => {
        // ELEMENT.Message.error(error.message);
        return Promise.reject(error);
    }
);

export function getBlob(url) {
    return service
        .get(url, {
            headers: {"Content-Type": "application/x-www-form-urlencoded"},
            responseType: "blob",
        })
        .then(async (data) => {
            const blob = new Blob([data]);
            return blob;
        });
}

export default service;
